@@ -1,6 +1,6 @@
 /* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
- * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -122,6 +122,29 @@ garcon_check_version (guint required_major,
 
 
 
+gchar **
+garcon_config_build_paths (const gchar *filename)
+{
+  const gchar * const *dirs;
+  gchar              **paths;
+  guint                n;
+
+  g_return_val_if_fail (filename != NULL && *filename != '\0', NULL);
+
+  dirs = g_get_system_config_dirs ();
+
+  paths = g_new0 (gchar *, 1 + g_strv_length ((gchar **)dirs) + 1);
+  
+  paths[0] = g_build_filename (g_get_user_config_dir (), filename, NULL);
+  for (n = 1; dirs[n-1] != NULL; ++n)
+    paths[n] = g_build_filename (dirs[n-1], filename, NULL);
+  paths[n] = NULL;
+
+  return paths;
+}
+
+
+
 /**
  * garcon_config_lookup:
  * @filename : relative filename of the config resource.
@@ -144,23 +167,25 @@ garcon_config_lookup (const gchar *filename)
 
   /* Look for the file in the user's config directory */
   path = g_build_filename (g_get_user_config_dir (), filename, NULL);
-  if (g_path_is_absolute (path)
-      && g_file_test (path, G_FILE_TEST_IS_REGULAR))
+  if (g_path_is_absolute (path) && g_file_test (path, G_FILE_TEST_IS_REGULAR))
     return path;
+    
   g_free (path);
+  path = NULL;
 
   /* Look for the file in the system config directories */
   dirs = g_get_system_config_dirs ();
-  for (i = 0; dirs[i] != NULL; ++i)
+  for (i = 0; path == NULL && dirs[i] != NULL; ++i)
     {
       /* Build the filename, if the file exists return the path */
       path = g_build_filename (dirs[i], filename, NULL);
-      if (g_path_is_absolute (path)
-          && g_file_test (path, G_FILE_TEST_IS_REGULAR))
-        return path;
-      g_free (path);
+      if (!g_path_is_absolute (path) || !g_file_test (path, G_FILE_TEST_IS_REGULAR))
+        {
+          g_free (path);
+          path = NULL;
+        }
     }
 
-  /* Nothing found */
-  return NULL;
+  /* Return the path or NULL if the file could not be found */
+  return path;
 }
